@@ -1,40 +1,60 @@
-#' Load taxonomic resources from either stable or current versions of APC and APNI
+#' @title Load taxonomic reference lists, APC & APNI
+#' 
+#' @description
+#' This function loads two taxonomic datasets for Australia's vascular plants, 
+#' the APC and APNI, into the global environment. It creates several data frames
+#' by filtering and selecting data from the loaded lists.
+#' 
+#' @details
+#' - It accesses taxonomic data from a dataset using the provided version number 
+#' or the default version.  
+#' - The output is several dataframes that include subsets of the APC/APNI based
+#' on taxon rank and taxonomic status.
 #'
-#' Loads taxonomic resources into the global environment. This function accesses taxonomic data from a dataset using the provided version number or the default version. The loaded data contains two lists: APC and APNI, which contain taxonomic information about plant species in Australia. The function creates several tibbles by filtering and selecting data from the loaded lists.
-#'
-#' @param stable_or_current_data Type of dataset to access. The default is "stable", which loads the
-#'   dataset from a github archived file. If set to "current", the dataset will be loaded from
-#'   a URL which is the cutting edge version, but this may change at any time without notice.
-#' @param version The version number of the dataset to use. Defaults to the default version.
-#'
-#' @param reload A logical indicating whether to reload the dataset from the data source. Defaults to FALSE.
+#' @param stable_or_current_data Type of dataset to access. 
+#' The default is "stable", which loads the dataset from a github archived file. 
+#' If set to "current", the dataset will be loaded from a URL which is the 
+#' cutting edge version, but this may change at any time without notice.
+#' @param version The version number of the dataset to use. 
+#' Defaults to the default version.
+#' 
+#' @param quiet A logical indicating whether to print status of loading to screen.
+#'  Defaults to FALSE.
 #'
 #' @return The taxonomic resources data loaded into the global environment.
 #' @export
 #'
 #' @examples
-#' \donttest{load_taxonomic_resources(stable_or_current_data="stable",version="0.0.2.9000")}
+#' \donttest{
+#' load_taxonomic_resources(stable_or_current_data="stable", 
+#' version="0.0.2.9000")}
 #'
-#' @importFrom dplyr filter select mutate distinct arrange
-#' @importFrom crayon red
 
 load_taxonomic_resources <-
   function(stable_or_current_data = "stable",
            version = default_version(),
-           reload = FALSE) {
-    message("Loading resources...", appendLF = FALSE)
-    on.exit(message("...done"))
+           quiet = FALSE) {
+    
+    
     
     taxonomic_resources <- dataset_access_function(
       version = version,
       path = tools::R_user_dir("APCalign"),
       type = stable_or_current_data
     )
-      
+    
+    
+    total_steps <- 3  # Define how many steps you expect in the function
+    pb <- utils::txtProgressBar(min = 0, max = total_steps, style = 2)
+    if(!quiet){
+      message("Loading resources into memory...")
+      utils::setTxtProgressBar(pb, 1)  
+    }
     if(is.null(taxonomic_resources)) {
       return(NULL)
     }
     
+    
     # Give list names
     names(taxonomic_resources) <- c("APC", "APNI")
     
@@ -42,62 +62,40 @@ load_taxonomic_resources <-
     ### Note: Use `zzzz zzzz` because the fuzzy matching algorithm can't handles NA's
     zzz <- "zzzz zzzz"
     
+    column_rename <- 
+      c(
+        taxon_ID = "taxonID",
+        taxon_rank = "taxonRank",
+        name_type = "nameType",
+        taxonomic_status = "taxonomicStatus",
+        pro_parte = "proParte",
+        scientific_name = "scientificName",
+        scientific_name_ID = "scientificNameID",
+        accepted_name_usage_ID = "acceptedNameUsageID",
+        accepted_name_usage = "acceptedNameUsage",
+        canonical_name = "canonicalName",
+        scientific_name_authorship = "scientificNameAuthorship",
+        taxon_rank_sort_order = "taxonRankSortOrder",
+        taxon_remarks = "taxonRemarks",
+        taxon_distribution = "taxonDistribution",
+        higher_classification = "higherClassification",
+        nomenclatural_code = "nomenclaturalCode",
+        dataset_name = "datasetName",
+        name_element = "nameElement"
+      )
+
     taxonomic_resources$APC <- taxonomic_resources$APC %>%
-      rename(
-        taxon_ID = .data$taxonID,
-        taxon_rank = .data$taxonRank,
-        name_type = .data$nameType,
-        taxonomic_status = .data$taxonomicStatus,
-        pro_parte = .data$proParte,
-        scientific_name = .data$scientificName,
-        scientific_name_ID = .data$scientificNameID,
-        accepted_name_usage_ID = .data$acceptedNameUsageID,
-        accepted_name_usage = .data$acceptedNameUsage,
-        canonical_name = .data$canonicalName,
-        scientific_name_authorship = .data$scientificNameAuthorship,
-        taxon_rank_sort_order = .data$taxonRankSortOrder,
-        taxon_remarks = .data$taxonRemarks,
-        taxon_distribution = .data$taxonDistribution,
-        higher_classification = .data$higherClassification,
-        nomenclatural_code = .data$nomenclaturalCode,
-        dataset_name = .data$datasetName
-      ) %>%
-      mutate(
+      dplyr::rename(dplyr::any_of(column_rename)) %>%
+      dplyr::mutate(
         genus = extract_genus(canonical_name),
-        taxon_rank = stringr::str_to_lower(taxon_rank),
-        taxon_rank = stringr::str_replace(taxon_rank, "regnum", "kingdom"),
-        taxon_rank = stringr::str_replace(taxon_rank, "classis", "class"),
-        taxon_rank = stringr::str_replace(taxon_rank, "ordo", "order"),
-        taxon_rank = stringr::str_replace(taxon_rank, "familia", "family"),
-        taxon_rank = stringr::str_replace(taxon_rank, "varietas", "variety"),
-        taxon_rank = stringr::str_replace(taxon_rank, "forma", "form"),
-        taxon_rank = stringr::str_replace(taxon_rank, "sectio", "section")
+        taxon_rank = standardise_taxon_rank(taxon_rank)
       )
     
     taxonomic_resources$APNI <- taxonomic_resources$APNI %>%
-      rename(
-        name_type = .data$nameType,
-        taxonomic_status = .data$taxonomicStatus,
-        taxon_rank = .data$taxonRank,
-        scientific_name = .data$scientificName,
-        scientific_name_ID = .data$scientificNameID,
-        canonical_name = .data$canonicalName,
-        scientific_name_authorship = .data$scientificNameAuthorship,
-        taxon_rank_sort_order = .data$taxonRankSortOrder,
-        nomenclatural_code = .data$nomenclaturalCode,
-        dataset_name = .data$datasetName,
-        name_element = .data$nameElement
-      )  %>%
-      mutate(
+      dplyr::rename(dplyr::any_of(column_rename)) %>%
+      dplyr::mutate(
         genus = extract_genus(canonical_name),
-        taxon_rank = stringr::str_to_lower(taxon_rank),
-        taxon_rank = stringr::str_replace(taxon_rank, "regnum", "kingdom"),
-        taxon_rank = stringr::str_replace(taxon_rank, "classis", "class"),
-        taxon_rank = stringr::str_replace(taxon_rank, "ordo", "order"),
-        taxon_rank = stringr::str_replace(taxon_rank, "familia", "family"),
-        taxon_rank = stringr::str_replace(taxon_rank, "varietas", "variety"),
-        taxon_rank = stringr::str_replace(taxon_rank, "forma", "form"),
-        taxon_rank = stringr::str_replace(taxon_rank, "sectio", "section")
+        taxon_rank = standardise_taxon_rank(taxon_rank)
       )
     
     APC_tmp <-
@@ -117,21 +115,24 @@ load_taxonomic_resources <-
         genus
       ) %>%
       dplyr::mutate(
-        # strip_names removes punctuation and filler words associated with infraspecific taxa (subsp, var, f, ser)
+        ## strip_names removes punctuation and filler words associated with
+        ## infraspecific taxa (subsp, var, f, ser)
         stripped_canonical = strip_names(canonical_name),
-        ## strip_names2 removes punctuation, filler words associated with infraspecific taxa (subsp, var, f, ser), and filler words associated with species name cases (x, sp)
-        ## strip_names2 is essential for the matches involving 2 or 3 words, since you want those words to not count filler words
-        stripped_canonical2 = strip_names_2(canonical_name),
+        ## strip_names_extra removes extra filler words associated with 
+        ## species name cases (x, sp)
+        ## strip_names_extra is essential for the matches involving 2 or 3 words,
+        ## since you want those words to not count filler words
+        stripped_canonical2 = strip_names_extra(stripped_canonical),
         stripped_scientific = strip_names(scientific_name),
         binomial = ifelse(
           taxon_rank == "species",
-          stringr::word(stripped_canonical2, start = 1, end = 2),
+          word(stripped_canonical2, start = 1, end = 2),
           zzz
         ),
         binomial = ifelse(is.na(binomial), zzz, binomial),
         binomial = base::replace(binomial, duplicated(binomial), zzz),
         genus = extract_genus(stripped_canonical),
-        trinomial = stringr::word(stripped_canonical2, start = 1, end = 3),
+        trinomial = word(stripped_canonical2, start = 1, end = 3),
         trinomial = ifelse(is.na(trinomial), zzz, trinomial),
         trinomial = base::replace(trinomial, duplicated(trinomial), zzz),
       ) %>%
@@ -147,6 +148,8 @@ load_taxonomic_resources <-
       dplyr::filter(taxonomic_status != "accepted") %>%
       dplyr::mutate(taxonomic_dataset = "APC")
     
+    
+    if(!quiet) utils::setTxtProgressBar(pb, 2) 
     # Repeated from above - bionomial, tronomials etc
     taxonomic_resources[["APNI names"]] <-
       taxonomic_resources$APNI %>%
@@ -161,15 +164,15 @@ load_taxonomic_resources <-
       dplyr::mutate(
         taxonomic_status = "unplaced for APC",
         stripped_canonical = strip_names(canonical_name),
-        stripped_canonical2 = strip_names_2(canonical_name),
+        stripped_canonical2 = strip_names_extra(stripped_canonical),
         stripped_scientific = strip_names(scientific_name),
         binomial = ifelse(
           taxon_rank == "species",
-          stringr::word(stripped_canonical2, start = 1, end = 2),
+          word(stripped_canonical2, start = 1, end = 2),
           "zzzz zzzz"
         ),
         binomial = ifelse(is.na(binomial), "zzzz zzzz", binomial),
-        trinomial = stringr::word(stripped_canonical2, start = 1, end = 3),
+        trinomial = word(stripped_canonical2, start = 1, end = 3),
         trinomial = ifelse(is.na(trinomial), "zzzz zzzz", trinomial),
         trinomial = base::replace(trinomial, duplicated(trinomial), "zzzz zzzz"),
         genus = extract_genus(stripped_canonical),
@@ -193,9 +196,10 @@ load_taxonomic_resources <-
         genus
       ) %>%
       dplyr::filter(taxon_rank %in% c("genus"), taxonomic_status == "accepted") %>%
+      dplyr::filter(!stringr::str_detect(genus, "aceae$")) %>%
       dplyr::mutate(taxonomic_dataset = "APC")
     
-    taxonomic_resources[["genera_known"]] <-
+    taxonomic_resources[["genera_synonym"]] <-
       taxonomic_resources$APC %>%
       dplyr::select(
         canonical_name,
@@ -211,9 +215,11 @@ load_taxonomic_resources <-
       ) %>%
       dplyr::filter(taxon_rank %in% c("genus")) %>%
       dplyr::filter(!canonical_name %in% taxonomic_resources$genera_accepted$canonical_name) %>%
+      dplyr::filter(!stringr::str_detect(genus, "aceae$")) %>%
       dplyr::mutate(taxonomic_dataset = "APC") %>%
       dplyr::distinct(canonical_name, .keep_all = TRUE)
     
+    if(!quiet) utils::setTxtProgressBar(pb, 3) 
     taxonomic_resources[["genera_APNI"]] <-
       taxonomic_resources$APNI %>%
       dplyr::select(
@@ -227,17 +233,18 @@ load_taxonomic_resources <-
       ) %>%
       dplyr::filter(taxon_rank %in% c("genus")) %>%
       dplyr::filter(!canonical_name %in% taxonomic_resources$APC$canonical_name) %>%
+      dplyr::filter(!stringr::str_detect(genus, "aceae$")) %>%
       dplyr::mutate(taxonomic_dataset = "APNI") %>%
       dplyr::distinct(canonical_name, .keep_all = TRUE)
     
     taxonomic_resources[["genera_all"]] <-
       dplyr::bind_rows(
         taxonomic_resources$genera_accepted,
-        taxonomic_resources$genera_known,
+        taxonomic_resources$genera_synonym,
         taxonomic_resources$genera_APNI
       ) %>%
       dplyr::mutate(
-        cleaned_name = stringr::word(accepted_name_usage, 1),
+        cleaned_name = word(accepted_name_usage, 1),
         cleaned_name = ifelse(is.na(cleaned_name), canonical_name, cleaned_name)
       ) %>%
       dplyr::distinct(cleaned_name, canonical_name, scientific_name, .keep_all = TRUE)
@@ -245,14 +252,35 @@ load_taxonomic_resources <-
     taxonomic_resources[["family_accepted"]] <-
       taxonomic_resources$APC %>%
       dplyr::filter(taxon_rank %in% c("family"), taxonomic_status == "accepted")
+
+    taxonomic_resources[["family_synonym"]] <-
+      taxonomic_resources$APC %>%
+      dplyr::select(
+        canonical_name,
+        accepted_name_usage,
+        accepted_name_usage_ID,
+        scientific_name,
+        taxonomic_status,
+        taxon_ID,
+        scientific_name_ID,
+        name_type,
+        taxon_rank,
+        genus
+      ) %>%
+      dplyr::filter(taxon_rank %in% c("family"), taxonomic_status != "accepted") %>%
+      dplyr::mutate(taxonomic_dataset = "APC") %>%
+      dplyr::distinct(canonical_name, .keep_all = TRUE)
     
+    close(pb)
+    if(!quiet) message("...done")
     return(taxonomic_resources)
   }
 
 ##' Access Australian Plant Census Dataset
 ##'
-##' This function provides access to the Australian Plant Census dataset containing information
-##' about various species. The dataset can be loaded from a github for a stable file or from a URL for the most cutting-edge, but not stable version.
+##' This function provides access to the Australian Plant Census dataset
+##' about various species. The dataset can be loaded from a github for a stable file or
+##' from a URL for the most cutting-edge, but not stable version.
 ##'
 ##' @param version Version number. The default is NULL, which will load the most recent
 ##'   version of the dataset on your computer or the most recent version known
@@ -263,8 +291,9 @@ load_taxonomic_resources <-
 ##'   delete the persistent data at any time by running `mydata_del(NULL)` (or
 ##'   `mydata_del(NULL, path)` if you use a different path).
 ##' @param type Type of dataset to access. The default is "stable", which loads the
-##'   dataset from a github archived file. If set to "current", the dataset will be loaded from
-##'   a URL which is the cutting edge version, but this may change at any time without notice.
+##'   dataset from a github archived file. If set to "current", the dataset will be
+##' loaded from a URL which is the cutting edge version, but this may change at any time
+##'  without notice.
 ##'
 ##' @examples
 ##'
@@ -298,7 +327,6 @@ dataset_access_function <-
       tryCatch({
         APC <- readr::read_csv(
           "https://biodiversity.org.au/nsl/services/export/taxonCsv",
-          n_max = 110000,
           col_types =
             readr::cols(
               .default = readr::col_character(),
@@ -312,7 +340,6 @@ dataset_access_function <-
         APNI <-
           readr::read_csv(
             "https://biodiversity.org.au/nsl/services/export/namesCsv",
-            n_max = 140000,
             col_types =
               readr::cols(
                 .default = readr::col_character(),
@@ -329,11 +356,6 @@ dataset_access_function <-
                 modified = readr::col_datetime(format = "")
               )
           )
-
-        on.exit(
-          close( "https://biodiversity.org.au/nsl/services/export/taxonCsv"),
-          close( "https://biodiversity.org.au/nsl/services/export/namesCsv")
-                )
       }, error = function(e) rlang::abort("Taxonomic resources not currently available, try again later")
       )
     }
@@ -350,9 +372,9 @@ dataset_access_function <-
 #' version is specified.
 #'
 #' @return A character string representing the default version for stable data.
+#' @example default_version()
 #'
-#'
-#' @noRd
+#' @export
 
 default_version <- function() {
   # Check if there is internet connection
@@ -376,9 +398,10 @@ default_version <- function() {
     
     response <- httr::GET(url)
   
-  if(httr::http_error(response)){
+  if(httr::http_error(response) | !network){
     message("API currently down, try again later")
-  } 
+    return(invisible(NULL))
+  } else
   release_data <- httr::content(response, "text") |> jsonlite::fromJSON()
   
   # Pull out versions
@@ -441,6 +464,7 @@ dataset_get <- function(version = default_version(),
   path_to_apni <- file.path(path, paste0("apni", version, ".parquet"))
   
   APC <- if (!file.exists(path_to_apc)) {
+    message("Downloading...")
     download_and_read_parquet(apc.url, path_to_apc)
   } else {
     arrow::read_parquet(path_to_apc)
